// IrDAClientDlg.cpp : implementation file
//

#include "stdafx.h"
#include "IrDAClient.h"
#include "IrDAClientDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CIrDAClientDlg dialog




CIrDAClientDlg::CIrDAClientDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CIrDAClientDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CIrDAClientDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LIST_DEVICES, lvDevices);
	DDX_Control(pDX, IDC_EDIT_TEXT, edText);
	DDX_Control(pDX, IDC_LIST_EVENTS, lbEvents);
	DDX_Control(pDX, IDC_EDIT_TIMEOUT, edTimeout);
	DDX_Control(pDX, IDC_EDIT_READ_BUFFER, edReadBuffer);
	DDX_Control(pDX, IDC_EDIT_WRITE_BUFFER, edWriteBuffer);
}

BEGIN_MESSAGE_MAP(CIrDAClientDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_BUTTON_DISCOVER, &CIrDAClientDlg::OnBnClickedButtonDiscover)
	ON_BN_CLICKED(IDC_BUTTON_CLEAR, &CIrDAClientDlg::OnBnClickedButtonClear)
	ON_BN_CLICKED(IDC_BUTTON_DISCONNECT, &CIrDAClientDlg::OnBnClickedButtonDisconnect)
	ON_BN_CLICKED(IDC_BUTTON_CONNECT, &CIrDAClientDlg::OnBnClickedButtonConnect)
	ON_BN_CLICKED(IDC_BUTTON_SEND, &CIrDAClientDlg::OnBnClickedButtonSend)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_BUTTON_SET_BUFFERS, &CIrDAClientDlg::OnBnClickedButtonSetBuffers)
END_MESSAGE_MAP()


// CIrDAClientDlg message handlers

BOOL CIrDAClientDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// TODO: Add extra initialization here

	lvDevices.InsertColumn(0, _T("Address"), 0, 200);
	lvDevices.InsertColumn(1, _T("Name"), 0, 200);
	lvDevices.SetExtendedStyle(lvDevices.GetExtendedStyle() | LVS_EX_FULLROWSELECT);

	edText.SetWindowText(_T("Send this text to selected client"));

	edTimeout.SetWindowText(_T("10000"));

	edReadBuffer.SetWindowText(_T("0"));
	edWriteBuffer.SetWindowText(_T("0"));

	__hook(&CwclIrDAClient::OnConnect, &wclIrDAClient, &CIrDAClientDlg::ClientConnect);
	__hook(&CwclIrDAClient::OnData, &wclIrDAClient, &CIrDAClientDlg::ClientData);
	__hook(&CwclIrDAClient::OnDisconnect, &wclIrDAClient, &CIrDAClientDlg::ClientDisconnect);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CIrDAClientDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CIrDAClientDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CIrDAClientDlg::ShowError(const int Error) const
{
	CString s;
	s.Format(_T("Error: 0x%.8X"), Error);
	AfxMessageBox(s);
}

void CIrDAClientDlg::OnBnClickedButtonDiscover()
{
	lvDevices.DeleteAllItems();

	wclIrDADevices Devices;
	int Res = wclIrDAReceiver.Discover(Devices);
	if (Res != WCL_E_SUCCESS)
		ShowError(Res);
	else
	{
		if (Devices.size() == 0)
			AfxMessageBox(_T("Nothing found"));
		else
		{
			for (size_t i = 0; i < Devices.size(); i++)
			{
				wclIrDADevice Device = Devices[i];
				
				CString s;
				s.Format(_T("%.8X"), Device.Address);
				lvDevices.InsertItem((int)i, s);
				lvDevices.SetItemText((int)i, 1, Device.Name.c_str());
			}
		}
	}
}

void CIrDAClientDlg::OnBnClickedButtonClear()
{
	lbEvents.ResetContent();
}

void CIrDAClientDlg::OnBnClickedButtonDisconnect()
{
	int Res = wclIrDAClient.Disconnect();
	if (Res != WCL_E_SUCCESS)
		ShowError(Res);
}

void CIrDAClientDlg::OnBnClickedButtonConnect()
{
	POSITION Pos = lvDevices.GetFirstSelectedItemPosition();
	if (Pos == NULL)
		AfxMessageBox(_T("Select device"));
	else
	{
		CString s = _T("0x") + lvDevices.GetItemText(lvDevices.GetNextSelectedItem(Pos), 0);
		unsigned long Addr = _tcstol(s.GetBuffer(), NULL, 16);
		edTimeout.GetWindowText(s);
		unsigned long Timeout = _ttoi(s);

		wclIrDAClient.Address = Addr;
		wclIrDAClient.Timeout = Timeout;

		int Res = wclIrDAClient.Connect();

		if (Res != WCL_E_SUCCESS)
			ShowError(Res);
	}
}

void CIrDAClientDlg::OnBnClickedButtonSend()
{
	CString Text;
	edText.GetWindowText(Text);
	CStringA Ansi(Text);
	unsigned long Sent = 0;
	int Res = wclIrDAClient.Write(Ansi.GetBuffer(), Ansi.GetLength(), Sent);
	if (Res != WCL_E_SUCCESS)
		ShowError(Res);
	else
	{
		Text.Format(_T("Send: %d from %d"), Sent, Ansi.GetLength());
		AfxMessageBox(Text);
	}
}

void CIrDAClientDlg::ClientConnect(void* Sender, const int Error)
{
	UNREFERENCED_PARAMETER(Sender);

	if (Error == WCL_E_SUCCESS)
	{
		lbEvents.AddString(_T("Connected"));

		CString s;

		// Try to get read and write buffers size.
		unsigned long Size = 0;
		int Res = wclIrDAClient.GetReadBufferSize(Size);
		if (Res != WCL_E_SUCCESS)
		{
			s.Format(_T("Get read buffer error: 0x%.8X"), Res);
			lbEvents.AddString(s);
		}
		else
		{
			s.Format(_T("%d"), Size);
			edReadBuffer.SetWindowText(s);
		}
		
		Size = 0;
		Res = wclIrDAClient.GetWriteBufferSize(Size);
		if (Res != WCL_E_SUCCESS)
		{
			s.Format(_T("Get write buffer error: 0x%.8X"), Res);
			lbEvents.AddString(s);
		}
		else
		{
			s.Format(_T("%d"), Size);
			edWriteBuffer.SetWindowText(s);
		}
	}
	else
	{
		CString s;
		s.Format(_T("Connect error: 0x%.8X"), Error);
		lbEvents.AddString(s);
	}
}

void CIrDAClientDlg::ClientData(void* Sender, const void* const Data, 
	const unsigned long Size)
{
	UNREFERENCED_PARAMETER(Sender);

	if (Size > 0)
	{
		CStringA Str((PCHAR)Data, Size);
		CString StrW(Str);
		CString a;
		a.Format(_T("Received: %s "), static_cast<LPCWSTR>(StrW));
		lbEvents.AddString(a);
	}
}

void CIrDAClientDlg::ClientDisconnect(void* Sender, const int Reason)
{
	UNREFERENCED_PARAMETER(Sender);

	CString s;
	s.Format(_T("Disconnected: 0x%.8X"), Reason);
	lbEvents.AddString(s);

	edReadBuffer.SetWindowText(_T("0"));
	edWriteBuffer.SetWindowText(_T("0"));
}

void CIrDAClientDlg::OnDestroy()
{
	CDialog::OnDestroy();

	wclIrDAClient.Disconnect();
	__unhook(&wclIrDAClient);
}

void CIrDAClientDlg::OnBnClickedButtonSetBuffers()
{
	CString s;

	// Try to set read buffer size first.
	edReadBuffer.GetWindowText(s);
	unsigned long Size = _ttoi(s);
	int Res = wclIrDAClient.SetReadBufferSize(Size);
	if (Res == WCL_E_SUCCESS)
		lbEvents.AddString(_T("Read buffer size changed."));
	else
	{
		s.Format(_T("Set read buffer size error: 0x%.8X"), Res);
		lbEvents.AddString(s);
	}
	
	// Now write buffer.
	edWriteBuffer.GetWindowText(s);
	Size = _ttoi(s);
	Res = wclIrDAClient.SetWriteBufferSize(Size);
	if (Res == WCL_E_SUCCESS)
		lbEvents.AddString(_T("Write buffer size changed."));
	else
	{
		s.Format(_T("Set write buffer size error: 0x%.8X"), Res);
		lbEvents.AddString(s);
	}
}
