// OppClientDlg.cpp : implementation file
//

#include "stdafx.h"
#include "OppClient.h"
#include "OppClientDlg.h"

#include "..\..\..\Source\Common\wclErrors.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CString IntToHex(const int i)
{
	CString s;
	s.Format(_T("%.8X"), i);
	return s;
}

unsigned long StrToInt(const CString& s)
{
	return _tcstoul(s, NULL, 16);
}

CString IntToStr(const int i)
{
	CString s;
	s.Format(_T("%d"), i);
	return s;
}


// COppClientDlg dialog




COppClientDlg::COppClientDlg(CWnd* pParent /*=NULL*/)
	: CDialog(COppClientDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void COppClientDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LIST_DEVICES, lvDevices);
	DDX_Control(pDX, IDC_EDIT_FILE_NAME, edFileName);
	DDX_Control(pDX, IDC_PROGRESS, ProgressBar);
	DDX_Control(pDX, IDC_LIST_LOG, lbLog);
}

BEGIN_MESSAGE_MAP(COppClientDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_BUTTON_CLEAR, &COppClientDlg::OnBnClickedButtonClear)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_BUTTON_DISCOVER, &COppClientDlg::OnBnClickedButtonDiscover)
	ON_BN_CLICKED(IDC_BUTTON_DISCONNECT, &COppClientDlg::OnBnClickedButtonDisconnect)
	ON_BN_CLICKED(IDC_BUTTON_CLOSE_SESSION, &COppClientDlg::OnBnClickedButtonCloseSession)
	ON_BN_CLICKED(IDC_BUTTON_OPEN_SESSION, &COppClientDlg::OnBnClickedButtonOpenSession)
	ON_BN_CLICKED(IDC_BUTTON_CONNECT, &COppClientDlg::OnBnClickedButtonConnect)
	ON_BN_CLICKED(IDC_BUTTON_ABORT, &COppClientDlg::OnBnClickedButtonAbort)
	ON_BN_CLICKED(IDC_BUTTON_SEND_FILE, &COppClientDlg::OnBnClickedButtonSendFile)
	ON_BN_CLICKED(IDC_BUTTON_GET_VCARD, &COppClientDlg::OnBnClickedButtonGetVcard)
	ON_BN_CLICKED(IDC_BUTTON_GET_CAPS, &COppClientDlg::OnBnClickedButtonGetCaps)
	ON_BN_CLICKED(IDC_BUTTON_SELECT_FILE, &COppClientDlg::OnBnClickedButtonSelectFile)
END_MESSAGE_MAP()


// COppClientDlg message handlers

BOOL COppClientDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// TODO: Add extra initialization here
	lvDevices.InsertColumn(0, _T("Address"), 0, 150);
	lvDevices.InsertColumn(1, _T("Name"), 0, 150);
	lvDevices.SetExtendedStyle(lvDevices.GetExtendedStyle() | LVS_EX_FULLROWSELECT);

	// We use ObjectPush Profile so must set the profile's UUID.
	wclIrDAClient.Service = _T("OBEX");
	wclIrDAClient.Mode = cmIrComm3Wire; // Required for OBEX
	
	FOpp = NULL;

	__hook(&CwclIrDAClient::OnDisconnect, &wclIrDAClient, &COppClientDlg::ClientDisconnect);
	__hook(&CwclIrDAClient::OnConnect, &wclIrDAClient, &COppClientDlg::ClientConnect);
	__hook(&CwclIrDAClient::OnDestroyProcessor, &wclIrDAClient, &COppClientDlg::ClientDestroyProcessor);
	__hook(&CwclIrDAClient::OnCreateProcessor, &wclIrDAClient, &COppClientDlg::ClientCreateProcessor);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void COppClientDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR COppClientDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}


void COppClientDlg::OnBnClickedButtonClear()
{
	lbLog.ResetContent();
}

void COppClientDlg::OnDestroy()
{
	CDialog::OnDestroy();

	wclIrDAClient.Disconnect();

	__unhook(&wclIrDAClient);
}

void COppClientDlg::OnBnClickedButtonDiscover()
{
	wclIrDADevices Devices;
	// Discover classic devices only.
	int Res = wclIrDAReceiver.Discover(Devices);
	if (Res != WCL_E_SUCCESS)
		AfxMessageBox(_T("Failed to start discovering: 0x") + IntToHex(Res));
	else
	{
		if (Devices.size() > 0)
		{
			for (wclIrDADevices::iterator i = Devices.begin(); i != Devices.end(); i++)
			{
				int Item = lvDevices.GetItemCount();
				lvDevices.InsertItem(Item, IntToHex((*i).Address));
				lvDevices.SetItemText(Item, 1, (*i).Name.c_str());
			}
		}
	}
}

void COppClientDlg::OnBnClickedButtonDisconnect()
{
	int Res = wclIrDAClient.Disconnect();
	if (Res != WCL_E_SUCCESS)
		AfxMessageBox(_T("Disconnect failed: 0x") + IntToHex(Res));
}

void COppClientDlg::ClientDisconnect(void* Sender, const int Reason)
{
	UNREFERENCED_PARAMETER(Sender);

	lbLog.AddString(_T("Disconnected with reason: 0x") + IntToHex(Reason));
}

void COppClientDlg::ClientConnect(void* Sender, const int Error)
{
	UNREFERENCED_PARAMETER(Sender);

	lbLog.AddString(_T("Connect. Operation result: 0x") + IntToHex(Error));
}

void COppClientDlg::ClientDestroyProcessor(void* Sender, 
	CwclClientDataConnection* const Connection)
{
	UNREFERENCED_PARAMETER(Sender);

	// Do we have any data processor created?
	if (FOpp != NULL)
	{
		// Make sure it is our connection is goind to destroy.
		if (Connection->Processor == FOpp)
		{
			// Ok, destroy the data processor here.
			__unhook(FOpp);
			delete FOpp;
			FOpp = NULL;
		}
	}
}

void COppClientDlg::ClientCreateProcessor(void* Sender, 
	CwclClientDataConnection* const Connection)
{
	UNREFERENCED_PARAMETER(Sender);

	// here we must create the data processor for the connection. In this demo
	// we use OPPClient.
	FOpp = new CwclObexOppClient(Connection);
	__hook(&CwclObexOppClient::OnConnect, FOpp, &COppClientDlg::OppConnect);
	__hook(&CwclObexOppClient::OnDisconnect, FOpp, &COppClientDlg::OppDisconnect);
	__hook(&CwclObexOppClient::OnGetComplete, FOpp, &COppClientDlg::OppGetComplete);
	__hook(&CwclObexOppClient::OnPutComplete, FOpp, &COppClientDlg::OppPutComplete);
	__hook(&CwclObexOppClient::OnProgress, FOpp, &COppClientDlg::OppProgress);
}

void COppClientDlg::OnBnClickedButtonCloseSession()
{
	if (FOpp == NULL)
		AfxMessageBox(_T("Not connected"));
	else
	{
		int Res = FOpp->Disconnect(_T(""));
		if (Res != WCL_E_SUCCESS)
			AfxMessageBox(_T("Close session failed: 0x") + IntToHex(Res));
	}
}

void COppClientDlg::OnBnClickedButtonOpenSession()
{
	if (FOpp == NULL)
		AfxMessageBox(_T("Not connected"));
	else
	{
		int Res = FOpp->Connect();
		if (Res != WCL_E_SUCCESS)
			AfxMessageBox(_T("Open session failed: 0x") + IntToHex(Res));
	}
}

void COppClientDlg::OppConnect(void* Sender, const int Error, 
	const tstring& Description)
{
	UNREFERENCED_PARAMETER(Sender);

	lbLog.AddString(_T("OBEX session opened with result: 0x") + IntToHex(Error) +
		_T(". Description: ") + CString(Description.c_str()));
}

void COppClientDlg::OppDisconnect(void* Sender, const int Error, 
	const tstring& Description)
{
	UNREFERENCED_PARAMETER(Sender);

	lbLog.AddString(_T("OBEX session closed with result: 0x") + IntToHex(Error) +
		_T(". Description: ") + CString(Description.c_str()));
}

void COppClientDlg::OnBnClickedButtonConnect()
{
	if (wclIrDAClient.State != csDisconnected)
		AfxMessageBox(_T("Client is connected"));
	else
	{
		POSITION Pos = lvDevices.GetFirstSelectedItemPosition();
		if (Pos == NULL)
			AfxMessageBox(_T("Select device"));
		else
		{
			int Item = lvDevices.GetNextSelectedItem(Pos);
			wclIrDAClient.Address = StrToInt(lvDevices.GetItemText(Item, 0));
			int Res = wclIrDAClient.Connect();
			if (Res != WCL_E_SUCCESS)
				AfxMessageBox(_T("Error: 0x") + IntToHex(Res));
		}
	}
}

void COppClientDlg::OppPutComplete(void* Sender, const int Error, 
	const tstring& Description, CwclStream* const Stream)
{
	UNREFERENCED_PARAMETER(Sender);

	lbLog.AddString(_T("File sending completed with result: 0x") + IntToHex(Error) +
		_T(". Description: ") + CString(Description.c_str()));
	
	ProgressBar.SetPos(0);
	ProgressBar.SetRange32(0, 0);
	
	// We must destroy the stream object here.
	delete Stream;
}

void COppClientDlg::OppProgress(void* Sender, const unsigned long Size, 
	const unsigned long Position)
{
	UNREFERENCED_PARAMETER(Sender);

	ProgressBar.SetRange32(0, Size);
	ProgressBar.SetPos(Position);
}

void COppClientDlg::OnBnClickedButtonAbort()
{
	if (FOpp == NULL)
		AfxMessageBox(_T("Not connected"));
	else
	{
		int Res = FOpp->Abort(_T(""));
		if (Res != WCL_E_SUCCESS)
			AfxMessageBox(_T("Open session failed: 0x") + IntToHex(Res));
	}
}

void COppClientDlg::OnBnClickedButtonSendFile()
{
	if (FOpp == NULL)
		AfxMessageBox(_T("Not connected"));
	else
	{
		CString s;		
		edFileName.GetWindowText(s);
		if (s == _T(""))
			AfxMessageBox(_T("Select file"));
		else
		{
			tstring FileName = tstring(s);
			CwclFileStream* Stream = new CwclFileStream(FileName, OPEN_EXISTING, GENERIC_READ,
				FILE_SHARE_READ);
			size_t last_slash_idx = FileName.find_last_of(_T("\\/"));
			if (tstring::npos != last_slash_idx)
				FileName.erase(0, last_slash_idx + 1);
			
			int Res = FOpp->Put(FileName, _T(""), Stream);
			if (Res != WCL_E_SUCCESS)
			{
				AfxMessageBox(_T("Send file failed: 0x") + IntToHex(Res));
				delete Stream;
			}
		}
	}
}

void COppClientDlg::OnBnClickedButtonGetVcard()
{
	if (FOpp == NULL)
		AfxMessageBox(_T("Not connected"));
	else
	{
		CwclFileStream* Stream = new CwclFileStream(_T("vcard.vcf"), CREATE_ALWAYS,
			GENERIC_WRITE, FILE_SHARE_WRITE);
		// Request default vCard object.
		int Res = FOpp->Get(_T("text/x-vCard"), Stream);
		if (Res != WCL_E_SUCCESS)
		{
			AfxMessageBox(_T("GetvCard failed: 0x") + IntToHex(Res));
			delete Stream;
		}
	}
}

void COppClientDlg::OnBnClickedButtonGetCaps()
{
	if (FOpp == NULL)
		AfxMessageBox(_T("Not connected"));
	else
	{
		CwclFileStream* Stream = new CwclFileStream(_T("caps.xml"), CREATE_ALWAYS,
			GENERIC_WRITE, FILE_SHARE_WRITE);
		// Request default vCard object.
		int Res = FOpp->Get(_T("x-obex/capability"), Stream);
		if (Res != WCL_E_SUCCESS)
		{
			AfxMessageBox(_T("GetCaps failed: 0x") + IntToHex(Res));
			delete Stream;
		}
	}
}

void COppClientDlg::OppGetComplete(void* Sender, const int Error, 
	const tstring& Description, CwclStream* const Stream)
{
	UNREFERENCED_PARAMETER(Sender);

	lbLog.AddString(_T("File receiving completed with result: 0x") +
		IntToHex(Error) + _T(". Description: ") + CString(Description.c_str()));
	
	ProgressBar.SetPos(0);
	ProgressBar.SetRange32(0, 0);
	
	// We must destroy the stream object here.
	delete Stream;
}

void COppClientDlg::OnBnClickedButtonSelectFile()
{
	CFileDialog Dlg(TRUE);
	if (Dlg.DoModal() == IDOK)
		edFileName.SetWindowText(Dlg.GetPathName());
}
