////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//   Wireless Communication Library 7                                         //
//                                                                            //
//   Copyright (C) 2006-2024 Mike Petrichenko                                 //
//                           Soft Service Company                             //
//                           All Rights Reserved                              //
//                                                                            //
//   http://www.btframework.com                                               //
//                                                                            //
//   support@btframework.com                                                  //
//   shop@btframework.com                                                     //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

#pragma once

#include <winsock.h>
#include <af_irda.h>
#include <initguid.h>

namespace wclIrDA
{
	namespace IrdaApi
	{
		DEFINE_GUID(IRDA_GUID, 0x6BDD1FC5, 0x810F, 0x11D0, 0xBE, 0xC7, 0x08, 0x00, 0x2B, 0xE2, 0x09, 0x2F);

		// Not from MS IrDA API
		#define LM_HB2_COMM		4
		#define LM_HB2_Message	8
		#define LM_HB2_HTTP		16
		#define LM_HB2_OBEX		32
		// =======

		/* Help functions. */

		/// <summary> Checks if an IrDA hardware is available. </summary>
		/// <returns> Returns <c>True</c> if the IrDA hardware is available. </returns>
		bool wclIsIrDAAvailable();

		/// <summary> Converts IrDA error code to WCL error code. </summary>
		/// <param name="WSAError"> The IrDA error code. </param>
		/// <returns> The WCL error code. </returns>
		int wclIrDADecodeWSAError(const int WSAError);
	}
}